--[[==========================================
	XPView
	==========================================
	An xp addon displaying statistics about your current experience gain.
  ]]--


XPView = {}
XPView.name = "XPView"
XPView.command = "/xpview"
XPView.version	= "v1.1.1"
XPView.settingsVersion = 2;
XPView.useVeteranPoints = false;

XPView.settingsDefaults = {
	font = "ZoFontAnnounceSmall";
	wndMain = {
		x = 10;
		y = 50;
		width = 170;
		height = 70;
	}
}

local colorToLevel = "33aa33";
local timeForNextUpdate = 0;
local xpCollected = 0; -- collected xp since last reset
local xpStep = 0;
local mobsToLevelup = 0;
local xpLastUpdate = GetUnitXP( "player" );
local xpStart = GetUnitXP( "player" ); -- xp at the time of last reset
local xpCurrent = GetUnitXP( "player" ); -- current player xp during cycle
local xpLevel = GetUnitXPMax( "player" ); -- maximum level xp
local timeStart = GetGameTimeMilliseconds(); -- time (s) of last reset
local timeElapsed = 0; -- temp
local xpLevelPercent = 0; -- temp
local xpPerHour = 0; -- temp
local xpText = " "; -- temp
local xpCollectedText = " (0xp)"; -- temp
local xpGain0 = 0;
local xpGain1 = 0;
local xpGain2 = 0;
local xpGainCur = 0;
local xpGainAvg = 0;
	

function XPView.Initialize( eventCode, addOnName )

	-- Only set up for XPView
	if ( addOnName ~= XPView.name ) then
		return
	end

	XPView.settings = ZO_SavedVars:New( "XPViewSettings" , XPView.settingsVersion, nil, XPView.settingsDefaults, nil );

	XPViewUI:SetAnchor( TOPLEFT, GuiRoot, TOPLEFT, XPView.settings.wndMain.x, XPView.settings.wndMain.y );	
	XPViewUI:SetWidth( XPView.settings.wndMain.width );
	XPViewUI:SetHeight( XPView.settings.wndMain.height );
	XPViewUIWindow:SetFont( XPView.settings.font );

	if ( GetUnitLevel( "player" ) == 50 ) then		
		XPView.useVeteranPoints = true;
		EVENT_MANAGER:RegisterForEvent( "XPView", EVENT_VETERAN_POINTS_UPDATE, XPView.UpdateXP );
	else
		EVENT_MANAGER:RegisterForEvent( "XPView", EVENT_EXPERIENCE_UPDATE, XPView.UpdateXP );
	end

	xpCurrent = XPView.GetPlayerXP();
	xpLastUpdate = XPView.GetPlayerXP();
	xpStart = XPView.GetPlayerXP();	
	xpLevel = XPView.GetPlayerXPMax();

	xpLevelPercent = xpCurrent/xpLevel * 100;			
	
	SLASH_COMMANDS[XPView.command] = XPViewSlashCommands

	XPView.UpdateUI();
		
	d( string.format( "XPView (%s) loaded.", XPView.version ) );
end

-- Hook initialization onto the ADD_ON_LOADED event
EVENT_MANAGER:RegisterForEvent( "XPView" , EVENT_ADD_ON_LOADED , XPView.Initialize )

function XPView.GetPlayerXP()
	if ( XPView.useVeteranPoints ) then
		return GetUnitVeteranPoints( "player" );
	else
		return GetUnitXP( "player" );
	end
end

function XPView.GetPlayerXPMax()
	if ( XPView.useVeteranPoints ) then
		return GetUnitVeteranPointsMax( "player" );
	else
		return GetUnitXPMax( "player" );
	end
end


-- Shortens large integers into ..k or ..m text
function XPView.FormatNumber( number )
	local numberText = "0";
	
	if ( number > 999999 ) then
		numberText = string.format( "%.2fm", number/1000000 );
	elseif ( number > 9999 ) then
		numberText = string.format( "%.1fk", number/1000 );
	else
		numberText = string.format( "%d", number );
	end
		
	return numberText;
end

function XPView.GetAsciiProgessString()
	local xpProgressString = "";	

	if ( xpLevelPercent < 20 ) then
		xpProgressString = "|> |c%x%x44%s (%d%%)|r ----->| |c%s%s|r";
	elseif ( xpLevelPercent < 40 ) then
		xpProgressString = "|-> |c%x%x44%s (%d%%)|r ---->| |c%s%s|r";
	elseif ( xpLevelPercent < 60 ) then
		xpProgressString = "|--> |c%x%x44%s (%d%%)|r --->| |c%s%s|r";
	elseif ( xpLevelPercent < 80 ) then
		xpProgressString = "|---> |c%x%x44%s (%d%%)|r -->| |c%s%s|r";
	elseif ( xpLevelPercent < 100 ) then
		xpProgressString = "|-----> |c%x%x44%s (%d%%)|r >| |c%s%s|r";
	end

	return xpProgressString;
end

function XPView.OnMoveStop( self )
	XPView.settings.wndMain.x = self:GetLeft();
	XPView.settings.wndMain.y = self:GetTop();
	XPView.settings.wndMain.width = self:GetWidth();
	XPView.settings.wndMain.height = self:GetHeight();
end

function XPView.UpdateUI()

	-- Format an "ascii" progress bar
	xpText = string.format( XPView.GetAsciiProgessString(), 200 - xpLevelPercent, 100 + xpLevelPercent, XPView.FormatNumber( xpCurrent ), xpLevelPercent, colorToLevel, XPView.FormatNumber( xpLevel ) );
	
	-- Set main window status text
	XPViewUIWindow:SetText( string.format( "|cffcc44%s|r xp/hour", XPView.FormatNumber( xpPerHour ) ) ..
			xpCollectedText ..
			"\n" ..
			string.format( "|cff8844%d:%02d|r / L^|c%s%d:%02d|r (|c88aa88%d|r)", timeElapsed/60, timeElapsed%60, colorToLevel, timeToLevel/60, timeToLevel%60, mobsToLevelup ) ..
			"\n" ..
			xpText
			);

	XPViewUIStatusbar:SetValue( xpLevelPercent );

end

function XPView.Update()

	-- check to see if we need to update (only update every 0.5 sec)
	if ( timeForNextUpdate < GetGameTimeMilliseconds() ) then

		timeForNextUpdate = GetGameTimeMilliseconds() + 500;
		
		-- get current time and calc time elapsed
		timeElapsed = GetGameTimeMilliseconds() - timeStart;

		-- convert to seconds
		timeElapsed = timeElapsed / 1000
		
		if ( timeElapsed <= 0 ) then
			timeElapsed = 0.01;
		end
		
		-- do a ring calculation
		if ( xpGainCur == 0 ) then
			xpGain0 = xpCollected / timeElapsed;
			xpGainCur = 1;
		elseif ( xpGainCur == 1 ) then
			xpGain1 = xpCollected / timeElapsed;
			xpGainCur = 2;
		elseif ( xpGainCur == 2 ) then
			xpGain2 = xpCollected / timeElapsed;
			xpGainCur = 0;
		end
		
		if ( xpGain1 == 0 ) then
			xpGain1 = xpGain0;
		end
		if ( xpGain2 == 0 ) then
			xpGain2 = xpGain1;
		end
		
		-- calc average gain over the last 3 gains recorded
		xpGainAvg = ( xpGain0 + xpGain1 + xpGain2 ) / 3;
		
		-- xp per hour
		xpPerHour = math.floor( xpGainAvg * 3600 );
		
		-- calc timeToLevel only with data
		if ( xpCollected > 0 ) then
			timeToLevel = ( xpLevel - xpCurrent ) / ( xpGainAvg );
		else
			timeToLevel = 0;
		end

		XPView.UpdateUI();
		
	end

end


function XPView.Reset()

	xpCollected = 0;
	xpCurrent = XPView.GetPlayerXP();
	xpStart = xpCurrent;	
	xpLevel = XPView.GetPlayerXPMax();
	timeStart = GetGameTimeMilliseconds();
	
	xpGain0 = 0;
	xpGain1 = 0;
	xpGain2 = 0;
	xpGainCur = 0;
	xpGainAvg = 0;

end


function XPView.UpdateXP( reason, unitTag, currentXP, maxXP )

	if ( unitTag ~= "player" ) then
		return;
	end

	-- check if we collected any xp this session. if not, reset on first xp encounter
	if ( xpCollected == 0 ) then
		timeStart = GetGameTimeMilliseconds();
	end
	
	-- check for level-up
	if ( currentXP > maxXP ) then
		xpStep = maxXP - xpLastUpdate;
	elseif ( maxXP ~= xpLevel ) then
		xpStep = xpStep + currentXP;
	else
		xpStep = currentXP - xpLastUpdate;
	end

	xpLastUpdate = currentXP;
	
	-- calculate # ob similar xp gains to levelup
	mobsToLevelup = math.ceil( ( maxXP - currentXP ) / xpStep );
	if ( mobsToLevelup < 0  or mobsToLevelup > 100000) then
		mobsToLevelup = 0;
	end

	xpCollected = xpCollected + xpStep;
		
	-- get player xp stuff
	xpCurrent = currentXP;
	xpLevel = maxXP;		
	
	-- check if xp data is invalid
	if ( xpCollected < 0 ) then
		XPView.Reset();
	end				
	
	-- format latest xp gain amount
	xpCollectedText = string.format( " (|cff8844%s|r xp) ", XPView.FormatNumber( xpStep ) )
	
	xpLevelPercent = xpCurrent/xpLevel * 100;
	
	XPView.UpdateUI();	

end

function XPViewSlashCommands( text )

	if ( text == "" ) then
		d( "XPView " .. XPView.version )
		d( "Command line options: " )
		d( "  hideBG" )
		d( "  reset" )
		return
	end

	if ( text == "hideBG" ) then
		XPViewUIBG:SetHidden( true );
		XPViewUIStatusbar:SetHidden( true );
		return
	end

	if ( text == "largeFonts" ) then
		XPView.settings.font = "ZoFontAnnounceMedium";
		XPViewUIWindow:SetFont( XPView.settings.font );
		XPViewUI:SetWidth( 250 );
		XPViewUI:SetHeight( 110 );
		XPView.OnMoveStop( XPViewUI );
		return
	end

	if ( text == "normalFonts" ) then
		XPView.settings.font = "ZoFontAnnounceSmall";
		XPViewUIWindow:SetFont( XPView.settings.font );
		XPViewUI:SetWidth( 170 );
		XPViewUI:SetHeight( 70 );
		XPView.OnMoveStop( XPViewUI );
		return
	end
		
	--[[if ( text == "test" ) then
		d( " " .. 50 .. " " .. GetUnitLevel( "player" ) );		
	end]]--

	if ( text == "reset" ) then
		XPView.Reset();
	end
end
